<?php
/**
  * CSV class
  * Basic CSV import/export
  * Usage: Set delimiter and quote character according to
  * current needs, i.e. it's possible (and often sensible)
  * to change delimiter and quote character between import
  * and export using the interface (setDelimiter, setQuote).
  * @author Jens Hatlak <jh@junetz.de>
  * @version 1.3 18/09/2007
  * @package Junetz
  */
  class CSV {
  /**
    * Field delimiter. Default: ';'
    * For tab separation set this to '\t'
    * @var string
    */
    var $delimiter;
  /**
    * Quote character. Default: '"'
    * @var string
    */
    var $quote;
  /**
    * No. of preview lines. Default: 5
    * @var int
    */
    var $prelines = 5;
  /**
    * No. of columns. Default: 0 (no limit)
    * @var int
    */
    var $cols = 0;
  /**
    * Display/return/export header line
    * @var boolean
    */
    var $head = true;
  /**
    * Line end character. Default: "\n"
    * @var string
    */
    var $lineend = "\n";
  /**
    * 2D Data array of lines (second dimension: fields).
    * @var array
    */
    var $data;

  /**
	* array field names
	* @var array
	*/
	var $fields;

  /**
    * Constructor
    * @param string $del Delimiter
    * @quote string $quote Quote character
    */
    function CSV($del=";", $quote='"', $lineend="\n") {
      $this->setDelimiter($del);
      $this->setQuote($quote);
      $this->setLineEnd($lineend);
      $this->data = array();
      $this->fields = array();
    }

  /**
    * Set Delimiter
    * @param string $del New delimiter
    */
    function setDelimiter($del) {
      if (!empty($del)) $this->delimiter = $del;
    }

  /**
    * Set Quote character
    * @param mixed $q New quote character (string or FALSE)
    */
    function setQuote($q) {
      if (empty($q))
        $q = "";
      else if (is_string($q))
        $this->quote = $q;
    }

  /**
    * Set no. of columns (for next import)
    * @param int $cols New no. of columns
    */
    function setCols($cols) {
      if (is_numeric($cols)) $this->cols = $cols;
    }

  /**
    * Set no. of preview lines
    * @param int $lines New no. of lines
    */
    function setPreLines($lines) {
      if (is_numeric($lines)) $this->prelines = $lines;
    }

  /**
    * Set heading switch.
    * Affects preview, reSort, exStream
    * @param boolean $head Activate heading switch
    */
    function setHead($head) {
      $this->head = (boolean) $head;
    }

  /**
    * Set line end character
    * @param string $lineend Line end character
    */
    function setLineEnd($lineend) {
      if (is_string($lineend)) $this->lineend = $lineend;
    }

  /**
    * Set field names
    * @array fields
    */
	function setFields() {
	  $this->fields = array_slice($this->data, 0, 1);
	  $this->fields = $this->fields[0];
	}

  /**
    * Get current delimiter
    * @return string Delimiter
    */
    function getDelimiter() {
      return $this->delimiter;
    }

  /**
    * Get current quote character
    * @return string Quote character
    */
    function getQuote() {
      return $this->quote;
    }

  /**
    * Get current column limit
    * @return int No. of cols
    */
    function getCols() {
      return $this->cols;
    }

  /**
    * Get current no. of preview lines
    * @return int No. of preview lines
    */
    function getPreLines() {
      return $this->prelines;
    }

  /**
    * Get current heading switch state
    * @return boolean Heading switch value
    */
    function getHead() {
      return $this->head;
    }

  /**
    * Get current line end character
    * @return string Line end character
    */
    function getLineEnd() {
      return $this->lineend;
    }


  /**
    * Get data
    * @return array 2D array of lines containing field arrays
    */
    function getData() {
      if ($this->head) return array_slice($this->data, 1);
      return $this->data;
    }

  /**
    * Get associative data
    * @return array 2D array of lines containing associative field arrays
    */
    function getDataAssoc() {
      if ($this->head)
      	$data =  array_slice($this->data, 1);
      else
      	$data = $this->data;

      foreach ($data as $n => $line)
      	foreach ($line as $i => $value)
      		$data_assoc[$n][$this->fields[$i]] = $value;
      return $data_assoc;
    }

  /**
    * Get field names
    * @return array field names
    */
    function getFields() {
		return $this->fields;
    }

  /**
    * Add a line containing fields as array or complete string
    * to the internal data (line) array
    * @param mixed Line contents
    */
    function addLine($data) {
      if (is_array($data)) $this->data[] = $data;
      else if (is_string($data)) {
        $this->data[] = $this->parseLine($data);
      }
    }

  /**
    * Parse a line string according to current settings
    * (delimiter/quote)
    * @return array Parsed data
    */
    function parseLine($str) {
      $data = explode($this->delimiter, $str);
      foreach ($data as $key=>$val)
        $data[$key] = str_replace($this->quote, '', $val);
      return $data;
    }

  /**
    * Returns the smaller value of
    * no. of preview lines and actual no. of lines.
    * Neither of the values is being altered
    * @return int Smaller value
    */
    function checkLineCount() {
      $reallines = count($this->data);
      if ($this->prelines>$reallines)
        return $reallines;
      return $this->prelines;
    }

  /**
    * Formats a line according to current settings
    * (delimiter/quote)
    * @param array $data Array of fields (strings)
    * @return string Formatted line
    */
    function formatLine($data) {
      foreach ($data as $str)
        $line .= sprintf('%s%s%s%s', $this->quote, $str,
                         $this->quote, $this->delimiter);
      return substr($line,0,-1).$this->lineend;
    }

  /**
    * Re-sort internal data array using field $field
    * @param int $field Field index
    * @param mixed $dir Direction (const. SORT_ASC or SORT_DESC)
    */
    function reSort($field, $dir=SORT_ASC) {
      if (is_numeric($field) && ($dir==SORT_ASC || $dir==SORT_DESC)) {
        foreach ($this->data as $key=>$val) {
          if ($this->head && $key==0) {
            // make sure heading is the first line
            if ($dir==SORT_ASC) $sortarray[] = -2147483647;
            else if ($dir==SORT_DESC) $sortarray[] = "zzz";
          } else
            $sortarray[] = $val[$field];
        }
        array_multisort($this->data, SORT_STRING, $sortarray, $dir);
      }
    }

  /**
    * Import data from arbitrary MySQL query
    * @param resource $res MySQL result resource
    */
    function queryImport($res) {
      $fc = mysql_num_fields($res);
      if ($fc==0) return;
      for ($i=0; $i < $fc; $i++)
        $data[] = mysql_field_name($res,$i);
      $this->addLine($data);
      while ($row = mysql_fetch_row($res)) {
        $data = array();
        for ($i=0; $i < $fc; $i++)
          $data[] = $row[$i];
        $this->addLine($data);
      }
    }

  /**
    * Export data from arbitrary MySQL query
    * @param resource $res MySQL result resource
    * @param string $name Preset file name
    * @param string $ext Extension (default: "csv")
    * @param boolean $nameContainsExt Wether $name contains $ext (default: FALSE)
    */
    function queryExport($res, $name, $ext, $nameContainsExt=false) {
      $fc = mysql_num_fields($res);
      if ($fc==0) return;
      $this->sendHeaders($name, $ext, $nameContainsExt);
      if ($this->head) {
        for ($i=0; $i < $fc; $i++)
          $data[] = mysql_field_name($res,$i);
        echo $this->formatLine($data);
      }
      while ($row = mysql_fetch_row($res))
        echo $this->formatLine($line);
      exit;
    }

  /**
    * Returns preview data
    * (up to no. of preview lines)
    * @return array 2D Array of lines containing fields
    */
    function preview() {
      if ($this->head) $start = 0;
      else $start = 1;
      for ($i=$start;$i<$this->checkLineCount();$i++)
        $data[] = $this->data[$i];
      return $data;
    }

  /**
    * Import uploaded file
    * @param string $field Name of fileselect field
    * @param int $length Optional maximal line length (default: 1024)
    */
    function uplImport($field, $length=1024) {
      if (!$GLOBALS["HTTP_POST_FILES"][$field]["error"])
        $this->fimport($GLOBALS["HTTP_POST_FILES"][$field]["tmp_name"], $length);
    }

  /**
    * Import file
    * @param string $file Name of file to be imported
    * @param int $length Optional maximal line length (default: 1024)
    */
    function fImport($file="", $length=1024) {
      if ($file!="" && file_exists($file)) {
        $fp = fopen($file,"r");
        while ($data = fgetcsv($fp, $length, $this->delimiter)) {
          if ($this->cols!=0)
            $data = array_slice($data, 0, $this->cols);
          $this->data[] = $data;
        }
	  $this->setFields();
      }
    }

  /**
    * Open export file stream (HTTP download)
    * @param string $name Preset file name
    * @param string $ext Extension (default: "csv")
    * @param boolean $nameContainsExt Wether $name contains $ext (default: FALSE)
    */
    function exStream($name, $ext="csv", $nameContainsExt=false) {
      if (empty($this->data)) return;
      $this->sendHeaders($name, $ext, $nameContainsExt);

      foreach ($this->data as $nr=>$line) {
        if ($this->head || $nr!=0)
          $datastr .= $this->formatLine($line);
      }

      // Dateigre fr Downloadzeit-Berechnung
      header("Content-Length: ".strlen($datastr));

      // Adressen ausgeben und Script beenden
      echo $datastr;
      exit;
    }

  /**
    * Send appropriate headers
    * @param string $name File name (may already contain extension,
    * in which case the second parameter is ignored)
    * @param string $ext Extension (default: "csv")
    */
    function sendHeaders($name, $ext="csv") {
      global $HTTP_USER_AGENT;
      header("Content-Type: text/text");
      if (!preg_match('/\.\w+$/', $name)) $name = "$name.$ext";
      header("Content-Disposition: ".
        (preg_match("/MSIE 5.5/", $HTTP_USER_AGENT)?"":"attachment; ").
        "filename=$name");
      header("Content-Description: PHP Generated Data");
      header("Content-Transfer-Encoding: binary");
      header("Cache-Control: post-check=0, pre-check=0");
      header("Connection: close");
    }
  }
?>